// Import the configuration file
import { getConfig, NodeConnection } from "./helper/config";
// Import SystemControl
import { SystemControl } from "./models/SystemController";
// Import the worker processes
import { DataWorker } from "./models/WorkerProcess";
// Import the mail model
import { sendMail } from "./models/MailModel";
// Import the file system library
import * as fs from "fs";
import * as path from "path";

// Global system control singleton
((global as any).logger as SystemControl) = new SystemControl();

/**
 * The main function to run the program with
 */
async function main() {
    // Get the configuration file
    const config = getConfig();

    // Initialize the Worker Nodes array
    let workerNodes : Array<DataWorker> = [];

    // Spawn in the DataWorkers
    for ( let i: number = 0; i < config.options.parallel_nodes; i++ ) {
        // Create a new data worker
        workerNodes.push(new DataWorker(config.options.parallel_node_tasks, config.options.database_export_path));
    }

    /**
     * Keep track of the worker that is assigned this node
     */
    let workerIndex = 0;
    // Start assigning nodes to the DataWorkers
    for ( let node of config.nodes ) {
        // Add the task to the node
        workerNodes[workerIndex].addNodeTask(node);
        // Increment the worker index
        workerIndex++;
        if ( workerIndex >= workerNodes.length ) workerIndex = 0;
    }

    const responseLogs: Map<string, string[]> = new Map<string, string[]>();

    // @TODO: Actually make these run in parallel
    for ( let w of workerNodes ) {

        const workerStartTime = new Date().getTime();
        let resolves: PromiseSettledResult<any>[] | string = await w.startProcessing();
        const workerEndTime: string = Number((workerStartTime - new Date().getTime()) / 1000).toFixed(2) + "s"

        let myLog: string[] = [
            `runTime: ${workerEndTime}`,
        ];

        for ( let r of resolves ) {
            myLog.push(JSON.stringify(r, null, 2));
        }

        responseLogs.set(w.getID(), myLog);
    }

    // Log to file
    if ( !!config.options.logs_path ) {
        // Log to a JSON file path
        const logFilePath: string = path.join(config.options.logs_path, new Date().getTime() + ".json");
        const dirPath = path.dirname(logFilePath);
        // Create the directory if it don't exist.
        if ( !fs.existsSync(dirPath) ) { fs.mkdirSync(dirPath); }

        // Write to file
        fs.writeFileSync(logFilePath, JSON.stringify(responseLogs as Object));
    }

    // Send the log via email
    if ( !!config.smtp ) {
        let htmlContent = `<h3>Database Log Report</h3><br>`;

        responseLogs.forEach( (value, key) => {
            htmlContent += `<h6>${value}<h6>`
            htmlContent += `<samp>${JSON.stringify(key, null, 4)}</samp>`
            htmlContent += "<hr>";
        })

        sendMail(htmlContent, config.smtp.to);
    }

}


// Run the main program
main();